==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webmail/client/inc/dragndrop.js <==
function cDnD(){
	storage.css('dragndrop');
	this.value;     	//draged data
	this.droper = [];   //drop target object stock

	this.startX;
	this.startY;
	this.addoffset = 5;

	this.old_docmove;
    this.old_docup;
    this.old_keydown;
    this.old_keyup;
    
    this._lastObj;		//last object with dragover (used for dragout)
};

/*
Main object methods:
 	_dragstart()
	_ondragend()

Target Object methods:
	_ondragover()
	_ondrop()
	_active_dropzone()


Drop data:
	type
    value
	originator
*/



/**
 * Registr drop objects
 **/
cDnD.prototype.registr_drop = function(obj, aType){
	if (typeof obj !='object' || !aType) return false;

	this.droper[obj._pathName]= aType;
	
	return true;
};

/**
 * @params:
 *	sHTML	- html content of draged block
 *	aData	- {type,arg,css,obj}
 *  x,y     - mouse coordinates
 **/
cDnD.prototype.create_drag = function (sHtml,aData,x,y){
	var me = this;

	if (typeof aData.type != 'string') return;

    aData.x = x;
    aData.y = y;

	if (aData.obj && aData.obj._ondragstart) aData.obj._ondragstart(aData);
	
	//create
	if (!this.eDiv){
		this.eContent = mkElement('div',{className:'dnd_body'});
		this.eDiv = mkElement('div',{id:'dnd_element'});
        this.eDiv.appendChild(mkElement('div',{className:'dnd_ico',innerHTML:'&nbsp;'}));
		this.eDiv.appendChild(this.eContent);
	}
	
	
	if (!this.eHide){
		this.eHide = mkElement('div',{id:'dnd_hide'});
		this.eHide.onclick = function(){me.remove_drag();};
	}

	this.eContent.innerHTML = sHtml;
	if (x) this.eDiv.style.left = (x-this.addoffset)+'px';
	if (y) this.eDiv.style.top = (y-this.addoffset)+'px';

	this.value = aData;

	if (!document.getElementById('dnd_hide'))
		document.getElementsByTagName('body')[0].appendChild(this.eHide);

	if (!document.getElementById('dnd_element'))
		document.getElementsByTagName('body')[0].appendChild(this.eDiv);


	//EVENT HANDLERS
	if (this.old_keydown != document.onkeydown)
		this.old_keydown = document.onkeydown;
	document.onkeydown = function (e){
		var e = e || window.event;

		if (me.eDiv && e.ctrlKey)
            addcss(me.eDiv,'ctrl');
	};

	if (this.old_keyup != document.onkeyup)
		this.old_keyup = document.onkeyup;
	document.onkeyup = function (e){
		var e = e || window.event;

		if (me.eDiv && !e.ctrlKey)
            removecss(me.eDiv,'ctrl');
	};

	if (this.old_docmove != document.onmousemove)
		this.old_docmove = document.onmousemove;
	document.onmousemove = function(e){
		var e = e || window.event;
		//try{
		me.value.x = e.clientX;
		me.value.y = e.clientY;
		//}catch(er){}

		me.eDiv.style.left = (e.clientX-me.addoffset)+'px';
		me.eDiv.style.top = (e.clientY-me.addoffset)+'px';
		
		//find Drop object
		var pos,obj;
		
		for (var i in me.droper){
			if (inArray(me.droper[i],me.value.type)>-1){
				try{
					obj = eval(i);
				}
				catch(er){
					delete me.droper[i];
					continue;
				}

				pos = getSize(obj._main);
				if (pos.x<=e.clientX && pos.x+pos.w>=e.clientX && pos.y<=e.clientY && pos.y+pos.h>=e.clientY){

					this._lastObj = i;

					if (obj._ondragover && obj._ondragover(me.value)){
						me.eHide.className = 'move';
						//addcss(me.eDiv,'move');
					}
					else{
						me.eHide.className = '';
						//removecss(me.eDiv,'move');
					}
				}
				else{
					if (this._lastObj && i == this._lastObj && obj._ondragout){
						obj._ondragout(me.value);
						this._lastObj = undefined;
					}
					
					me.eHide.className = '';
					//removecss(me.eDiv,'move');
				}
			}
			else{
				me.eHide.className = '';
				//removecss(me.eDiv,'move');
			}
		}
	};

	//MOUSEUP
    if (this.old_docup != document.onmouseup)
		this.old_docup = document.onmouseup;
	document.onmouseup = function(e){
		var e = e || window.event;

		me.value.x = e.clientX;
		me.value.y = e.clientY;
		me.value.ctrl = e.ctrlKey;
		var ok = false;

		//find Drop object
		for (var i in me.droper){
			if (inArray(me.droper[i],me.value.type)>-1){
				try{
					obj = eval(i);
				}
				catch(er){
					delete me.droper[i];
					continue;
				}

				if (!obj._ondrop) continue;

				pos = getSize(obj._main);
				if (pos.x<=e.clientX && pos.x+pos.w>=e.clientX && pos.y<=e.clientY && pos.y+pos.h>=e.clientY){
					ok = obj._ondrop(me.value);
					break;
				}
			}
		}

		me.remove_drag(!ok);
	};

	this.active_drop(1);
};

/**
 * Activate dropzones if defined in drop objects
 * @option: b - true for Active, false for Deactive
 **/
cDnD.prototype.active_drop = function (b){
	for (var i in this.droper){
		if (inArray(this.droper[i],this.value.type)>-1){
			var obj = eval(i);
			if (obj._active_dropzone)
				obj._active_dropzone(b?this.value:undefined);
		}
	}
};

/**
 * Remove draged element, disable drop zones
 **/
cDnD.prototype.remove_drag = function (){
	if (this.eDiv){

		//revert onmousemove
		if (this.old_docmove != document.onmousemove)
			document.onmousemove = this.old_docmove || null;
			this.old_docmove = null;

		//revert onmouseup
		if (this.old_docup != document.onmouseup)
			document.onmouseup = this.old_docup || null;
			this.old_docup = null;

		//revert onkeydown
		if (this.old_keydown != document.onkeydown)
        	document.onkeydown = this.old_keydown || null;
        	this.old_keydown = null;

		//revert onkeydown
		if (this.old_keyup != document.onkeyup)
        	document.onkeyup = this.old_keyup || null;
        	this.old_keyup = null;

		//remove child
		try{
			this.eDiv.parentNode.removeChild(this.eDiv);
			this.eHide.parentNode.removeChild(this.eHide);
		}
		catch(er){}

		//disable dropzones
		this.active_drop();
		
		if (this.value && this.value.obj && this.value.obj._ondragend)
			this.value.obj._ondragend();
			
		this.value = undefined;
	}
	this.eDiv = undefined;
	this._lastObj = undefined;
};

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webmail/client/inc/dragndrop.js <==
function cDnD(){
	storage.css('dragndrop');
	this.value;     	//draged data
	this.droper = [];   //drop target object stock

	this.startX;
	this.startY;
	this.addoffset = 5;

	this.old_docmove;
    this.old_docup;
    this.old_keydown;
    this.old_keyup;
    
    this._lastObj;		//last object with dragover (used for dragout)
};

/*
Main object methods:
 	_dragstart()
	_ondragend()

Target Object methods:
	_ondragover()
	_ondrop()
	_active_dropzone()


Drop data:
	type
    value
	originator
*/



/**
 * Registr drop objects
 **/
cDnD.prototype.registr_drop = function(obj, aType){
	if (typeof obj !='object' || !aType) return false;

	this.droper[obj._pathName] = aType;
	
	return true;
};

/**
 * @params:
 *	sHTML	- html content of draged block
 *	aData	- {type,arg,css,obj}
 *  x,y     - mouse coordinates
 **/
cDnD.prototype.create_drag = function (sHtml,aData,x,y){

	var me = this;

	if (typeof aData.type != 'string') return;

    aData.x = x;
    aData.y = y;

	if (aData.obj && aData.obj._ondragstart) aData.obj._ondragstart(aData);
	
	//create
	if (!this.eDiv)
		this.eDiv = mkElement('div',{id:'dnd_element'});

	if (!this.eHide){
		this.eHide = mkElement('div',{id:'dnd_hide'});
		this.eHide.onclick = function(){me.remove_drag();};
	}

	if (!document.getElementById('dnd_hide'))
		document.getElementsByTagName('body')[0].appendChild(this.eHide);

	
	if (x) this.eDiv.style.left = (x-this.addoffset)+'px';
	if (y) this.eDiv.style.top = (y-this.addoffset)+'px';
	this.eDiv.innerHTML = '<div class="dnd_ico">&nbsp;</div><div class="dnd_body">'+ sHtml +'</div>';
	
	if (!document.getElementById('dnd_element'))
		document.getElementsByTagName('body')[0].appendChild(this.eDiv);

	this.value = aData;


	//EVENT HANDLERS
	this.__keydown = function (e){
		var e = e || window.event;

		if (me.eDiv && e.ctrlKey)
            addcss(me.eDiv,'ctrl');
	};
    gui._obeyEvent('keydown',[this,'__keydown']);
    
	this.__keyup = function (e){
		var e = e || window.event;

		if (me.eDiv && !e.ctrlKey)
            removecss(me.eDiv,'ctrl');
	};
    gui._obeyEvent('keyup',[this,'__keyup']);

    //MOUSEMOVE
	this.__mousemove = function(e){

		me.value.x = e.clientX;
		me.value.y = e.clientY;

		me.eDiv.style.left = (e.clientX-me.addoffset)+'px';
		me.eDiv.style.top = (e.clientY-me.addoffset)+'px';
		
		//find Drop object
		var pos,obj;
		
		for (var i in me.droper){
			if (inArray(me.droper[i],me.value.type)>-1){
				try{
					obj = eval(i);
				}
				catch(er){
					delete me.droper[i];
					continue;
				}

				pos = getSize(obj._main);
				if (pos.x<=e.clientX && pos.x+pos.w>=e.clientX && pos.y<=e.clientY && pos.y+pos.h>=e.clientY){

					this._lastObj = i;

					if (obj._ondragover && obj._ondragover(me.value))
						me.eHide.className = 'move';
					else
						me.eHide.className = '';
						
					break;
				}
				else{
					if (this._lastObj && i == this._lastObj && obj._ondragout){
						obj._ondragout(me.value);
						this._lastObj = undefined;
					}
					
					me.eHide.className = '';
				}
			}
			else
				me.eHide.className = '';
		}
	};
    gui._obeyEvent('mousemove',[this,'__mousemove']);

	//MOUSEUP
	this.__mouseup = function(e){

		me.value.x = e.clientX;
		me.value.y = e.clientY;
		me.value.ctrl = e.ctrlKey;
		var ok = false;

		//find Drop object
		for (var i in me.droper){
			if (inArray(me.droper[i],me.value.type)>-1){
				try{
					obj = eval(i);
				}
				catch(er){
					delete me.droper[i];
					continue;
				}

				if (!obj._ondrop) continue;

				pos = getSize(obj._main);
				if (pos.x<=e.clientX && pos.x+pos.w>=e.clientX && pos.y<=e.clientY && pos.y+pos.h>=e.clientY){
					ok = obj._ondrop(me.value);
					break;
				}
			}
		}

		me.remove_drag();
	};
	gui._obeyEvent('mouseup',[this,'__mouseup']);

	this.active_drop(1);
};

/**
 * Activate dropzones if defined in drop objects
 * @option: b - true for Active, false for Deactive
 **/
cDnD.prototype.active_drop = function (b){
	for (var i in this.droper){
		if (inArray(this.droper[i],this.value.type)>-1){
			try{
				var obj = eval(i);
				if (obj._active_dropzone)
					obj._active_dropzone(b?this.value:undefined);
			}
			catch(e){
                delete this.droper[i];
			}
		}
	}
};

/**
 * Remove draged element, disable drop zones
 **/
cDnD.prototype.remove_drag = function (){
	if (this.eDiv){

		gui._disobeyEvent('mousemove',[this,'__mousemove']);
		gui._disobeyEvent('mouseup',[this,'__mouseup']);
		gui._disobeyEvent('keydown',[this,'__keydown']);
		gui._disobeyEvent('keyup',[this,'__keyup']);

		//remove child
		try{
			this.eDiv.parentNode.removeChild(this.eDiv);
			this.eHide.parentNode.removeChild(this.eHide);
		}
		catch(er){}

		//disable dropzones
		this.active_drop();
		
		if (this.value && this.value.obj && this.value.obj._ondragend)
			this.value.obj._ondragend();
			
		this.value = undefined;
	}
	this.eDiv = undefined;
	this._lastObj = undefined;
};
